@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2002 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2002 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <xfs/libxfs.h>
@@ -48,9 +34,7 @@
 #include "media.h"
 #include "content.h"
 #include "content_inode.h"
-#ifdef DMEXTATTR
 #include "hsmapi.h"
-#endif /* DMEXTATTR */
 
 #define MACROBITS
 #include "inomap.h"
@@ -72,9 +56,7 @@
 
 extern bool_t preemptchk( int );
 extern size_t pgsz;
-#ifdef DMEXTATTR
 extern hsm_fs_ctxt_t *hsm_fs_ctxtp;
-#endif /* DMEXTATTR */
 extern u_int64_t maxdumpfilesize;
 extern bool_t allowexcludefiles_pr;
 
@@ -735,8 +717,17 @@ cb_add( void *arg1,
 				return 0;
 			}
 
-#ifdef EXTATTR
-			if (allowexcludefiles_pr && (statp->bs_xflags & XFS_XFLAG_HASATTR)) {
+			if (allowexcludefiles_pr && statp->bs_xflags & XFS_XFLAG_NODUMP) {
+				mlog( MLOG_DEBUG | MLOG_EXCLFILES,
+				      "pruned ino %llu, owner %u, estimated size %llu: skip flag set\n",
+				      statp->bs_ino,
+				      statp->bs_uid,
+				      estimated_size );
+				map_add( ino, MAP_NDR_NOCHNG );
+				inomap_exclude_skipattr++;
+				return 0;
+			}
+			else if (allowexcludefiles_pr && (statp->bs_xflags & XFS_XFLAG_HASATTR)) {
 				int rval;
 				attr_multiop_t attrop;
 				static char *skip_attr_name = "SGI_XFSDUMP_SKIP_FILE";
@@ -764,7 +755,6 @@ cb_add( void *arg1,
 					return 0;
 				}
                         }
-#endif /* EXTATTR */
 
 			map_add( ino, MAP_NDR_CHANGE );
 #ifdef DEBUG_INOMAP
@@ -2158,7 +2148,6 @@ quantity2offset( jdm_fshandle_t *fshandlep, xfs_bstat_t *statp, off64_t qty )
 	off64_t offset_next;
 	off64_t qty_accum;
 
-#ifdef DMEXTATTR
 	/* If GETOPT_DUMPASOFFLINE was specified and the HSM provided an
 	 * estimate, then use it.
 	 */
@@ -2167,7 +2156,6 @@ quantity2offset( jdm_fshandle_t *fshandlep, xfs_bstat_t *statp, off64_t qty )
 		if (HsmEstimateFileOffset(hsm_fs_ctxtp, statp, qty, &offset))
 			return offset;
 	}
-#endif /* DMEXTATTR */
 
 	offset = 0;
 	offset_next = 0;
@@ -2234,14 +2222,12 @@ estimate_dump_space( xfs_bstat_t *statp )
 		/* very rough: must improve this.  If GETOPT_DUMPASOFFLINE was
 		 * specified and the HSM provided an estimate, then use it.
 		 */
-#ifdef DMEXTATTR
 		if (hsm_fs_ctxtp) {
 			off64_t	bytes;
 
 			if (HsmEstimateFileSpace(hsm_fs_ctxtp, statp, &bytes))
 				return bytes;
 		}
-#endif	/* DMEXTATTR */
 		return statp->bs_blocks * ( off64_t )statp->bs_blksize;
 	case S_IFIFO:
 	case S_IFCHR:
