@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2002 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2002 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*
@@ -72,6 +58,8 @@
 #include <attr/attributes.h>
 #include <xfs/xfs_dfrag.h>
 
+char *progname;
+
 int vflag;
 int gflag;
 static int Mflag;
@@ -106,7 +94,6 @@ static __int64_t	minimumfree = 2048;
 static time_t howlong = 7200;		/* default seconds of reorganizing */
 static char *leftofffile = "/var/tmp/.fsrlast_xfs";/* where we left off last */
 static char *mtab = MOUNTED;
-static char *progname;
 static time_t endtime;
 static time_t starttime;
 static xfs_ino_t	leftoffino = 0;
@@ -174,14 +161,14 @@ xfs_bulkstat_single(int fd, xfs_ino_t *lastip, xfs_bstat_t *ubuffer)
 
 int 
 xfs_bulkstat(int fd, xfs_ino_t *lastip, int icount, 
-                    xfs_bstat_t *ubuffer, size_t *ocount)
+                    xfs_bstat_t *ubuffer, __s32 *ocount)
 {
     xfs_fsop_bulkreq_t  bulkreq;
     
     bulkreq.lastip = lastip;
     bulkreq.icount = icount;
     bulkreq.ubuffer = ubuffer;
-    bulkreq.ocount =  (__s32 *)ocount;
+    bulkreq.ocount = ocount;
     return ioctl(fd, XFS_IOC_FSBULKSTAT, &bulkreq);
 }
 
@@ -653,7 +640,7 @@ fsrfs(char *mntdir, xfs_ino_t startino, int targetrange)
 	int	fsfd, fd;
 	int	count = 0;
 	int	ret;
-	size_t buflenout;
+	__s32	buflenout;
 	xfs_bstat_t buf[GRABSZ];
 	char	fname[64];
 	char	*tname;
@@ -1080,11 +1067,11 @@ packfile(char *fname, char *tname, int fd, xfs_bstat_t *statp, int do_rt)
 		unlink(ffname);
 	}
 
-	/* Loop through block map copying the file. */
+	/* Loop through block map allocating new extents */
 	for (extent = 0; extent < nextents; extent++) {
 		pos = outmap[extent].bmv_offset;
 		if (outmap[extent].bmv_block == -1) {
-			space.l_whence = 0;
+			space.l_whence = SEEK_SET;
 			space.l_start = pos;
 			space.l_len = outmap[extent].bmv_length;
 			if (ioctl(tfd, XFS_IOC_UNRESVSP64, &space) < 0) {
@@ -1092,7 +1079,6 @@ packfile(char *fname, char *tname, int fd, xfs_bstat_t *statp, int do_rt)
 					   tname);
 			}
 			lseek64(tfd, outmap[extent].bmv_length, SEEK_CUR);
-			lseek64(fd, outmap[extent].bmv_length, SEEK_CUR);
 			continue;
 		} else if (outmap[extent].bmv_length == 0) {
 			/* to catch holes at the beginning of the file */
@@ -1110,6 +1096,38 @@ packfile(char *fname, char *tname, int fd, xfs_bstat_t *statp, int do_rt)
 				free(fbuf);
 				return -1;
 			}
+			lseek64(tfd, outmap[extent].bmv_length, SEEK_CUR);
+		}
+	} /* end of space allocation loop */
+
+	if (lseek64(tfd, 0, SEEK_SET)) {
+		fsrprintf(_("Couldn't rewind on temporary file\n"));
+		close(tfd);
+		free(fbuf);
+		return -1;
+	}
+
+	/* Check if the temporary file has fewer extents */
+	new_nextents = getnextents(tfd);
+	if (dflag)
+		fsrprintf(_("Temporary file has %d extents (%d in original)\n"), new_nextents, cur_nextents);
+	if (cur_nextents <= new_nextents) {
+		if (vflag)
+			fsrprintf(_("No improvement will be made (skipping): %s\n"), fname);
+		close(tfd);
+		return 1; /* no change/no error */
+	}
+
+	/* Loop through block map copying the file. */
+	for (extent = 0; extent < nextents; extent++) {
+		pos = outmap[extent].bmv_offset;
+		if (outmap[extent].bmv_block == -1) {
+			lseek64(tfd, outmap[extent].bmv_length, SEEK_CUR);
+			lseek64(fd, outmap[extent].bmv_length, SEEK_CUR);
+			continue;
+		} else if (outmap[extent].bmv_length == 0) {
+			/* to catch holes at the beginning of the file */
+			continue;
 		}
 		for (cnt = outmap[extent].bmv_length; cnt > 0;
 		     cnt -= ct, pos += ct) {
@@ -1195,15 +1213,6 @@ packfile(char *fname, char *tname, int fd, xfs_bstat_t *statp, int do_rt)
 	sx.sx_offset   = 0;
 	sx.sx_length   = statp->bs_size;
 
-	/* Check if the extent count improved */
-	new_nextents = getnextents(tfd);
-	if (cur_nextents <= new_nextents) {
-		if (vflag)
-			fsrprintf(_("No improvement made: %s\n"), fname);
-		close(tfd);
-		return 1; /* no change/no error */
-	}
-
 	/* switch to the owner's id, to keep quota in line */
         if (fchown(tfd, statp->bs_uid, statp->bs_gid) < 0) {
                 if (vflag)
