@@ -1,33 +1,34 @@
 /*
- * Copyright (c) 2000-2001 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2001 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/*
+ * Directory attributes are written on tape during the directory dump phase,
+ * which precedes the non-dir dump phase. The directory attributes cannot be
+ * restored, however, until all of the non-dir files have been restored
+ * (directory timestamps would be wrong, directory inherit flags would
+ * interfere, etc.) These routines allow the directory attributes to be stored
+ * on disk when they are read from the dump stream so that they can be applied
+ * at a later time.
+ *
+ * Despite the name, these routines are not used solely for directory attributes
+ * anymore. Files which use the immutable flag and have extended attributes run
+ * into a problem similar to that of directories -- some of the attributes
+ * cannot be set at the time the attributes are read from the dump.
  */
 
 #include <xfs/libxfs.h>
@@ -115,18 +116,15 @@ struct dirattr {
 	time32_t d_ctime;
 	u_int32_t d_xflags;
 	u_int32_t d_extsize;
+	u_int32_t d_projid;
 	u_int32_t d_dmevmask;
 	u_int32_t d_dmstate;
-#ifdef EXTATTR
 	off64_t d_extattroff;
-#endif /* EXTATTR */
 };
 
 typedef struct dirattr dirattr_t;
 
-#ifdef EXTATTR
 #define DIRATTR_EXTATTROFFNULL	( ( off64_t )OFF64MAX )
-#endif /* EXTATTR */
 
 /* dirattr persistent context definition
  */
@@ -146,11 +144,9 @@ struct dirattr_tran {
 	bool_t dt_at_endpr;
 	dah_t dt_cachedh;
 	dirattr_t dt_cached_dirattr;
-#ifdef EXTATTR
 	char *dt_extattrpathname;
 	int dt_extattrfd;
 	bool_t dt_extattrfdbadpr;
-#endif /* EXTATTR */
 };
 
 typedef struct dirattr_tran dirattr_tran_t;
@@ -179,9 +175,7 @@ extern size_t pgsz;
 /* forward declarations of locally defined static functions ******************/
 
 static void dirattr_get( dah_t );
-#ifdef EXTATTR
 static void dirattr_flush( void );
-#endif /* EXTATTR */
 #ifdef DIRATTRCHK
 static u_int16_t calcdixcum( dix_t dix );
 #endif /* DIRATTRCHK */
@@ -193,9 +187,7 @@ static u_int16_t calcdixcum( dix_t dix );
 /* definition of locally defined static variables *****************************/
 
 static char *dirattrfile = "dirattr";
-#ifdef EXTATTR
 static char *dirextattrfile = "dirextattr";
-#endif /* EXTATTR */
 static dirattr_tran_t *dtp = 0;
 static dirattr_pers_t *dpp = 0;
 
@@ -223,9 +215,7 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 	ASSERT( dtp );
 	dtp->dt_cachedh = DAH_NULL;
 	dtp->dt_fd = -1;
-#ifdef EXTATTR
 	dtp->dt_extattrfd = -1;
-#endif /* EXTATTR */
 
 	/* generate a string containing the pathname of the dirattr file
 	 */
@@ -268,7 +258,7 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 		 */
 		{
 		bool_t successpr;
-		intgen_t ioctlcmd;
+		unsigned int ioctlcmd;
 		intgen_t loglevel;
 		size_t trycnt;
 
@@ -345,7 +335,6 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 	 */
 	dtp->dt_at_endpr = BOOL_FALSE;
 
-#ifdef EXTATTR
 	/* calculate the dir extattr pathname, and set the fd to -1.
 	 * file will be created on demand.
 	 */
@@ -355,7 +344,6 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 	if ( resume ) {
 		( void )unlink( dtp->dt_extattrpathname );
 	}
-#endif /* EXTATTR */
 
 	return BOOL_TRUE;
 }
@@ -382,7 +370,6 @@ dirattr_cleanup( void )
 		( void )unlink( dtp->dt_pathname );
 		free( ( void * )dtp->dt_pathname );
 	}
-#ifdef EXTATTR
 	if ( dtp->dt_extattrfd >= 0 ) {
 		( void )close( dtp->dt_extattrfd );
 		dtp->dt_extattrfd = -1;
@@ -391,7 +378,7 @@ dirattr_cleanup( void )
 		( void )unlink( dtp->dt_extattrpathname );
 		free( ( void * )dtp->dt_extattrpathname );
 	}
-#endif /* EXTATTR */
+
 	free( ( void * )dtp );
 	dtp = 0;
 }
@@ -444,6 +431,7 @@ dirattr_add( filehdr_t *fhdrp )
 	dirattr.d_ctime = ( time32_t )fhdrp->fh_stat.bs_ctime.tv_sec;
 	dirattr.d_xflags = fhdrp->fh_stat.bs_xflags;
 	dirattr.d_extsize = ( u_int32_t )fhdrp->fh_stat.bs_extsize;
+	dirattr.d_projid = fhdrp->fh_stat.bs_projid;
 	dirattr.d_dmevmask = fhdrp->fh_stat.bs_dmevmask;
 	dirattr.d_dmstate = ( u_int32_t )fhdrp->fh_stat.bs_dmstate;
 #ifdef DIRATTRCHK
@@ -451,9 +439,7 @@ dirattr_add( filehdr_t *fhdrp )
 	sum = calcdixcum( dix );
 	dirattr.d_sum = sum;
 #endif /* DIRATTRCHK */
-#ifdef EXTATTR
 	dirattr.d_extattroff = DIRATTR_EXTATTROFFNULL;
-#endif /* EXTATTR */
 
 	/* write the dirattr
 	 */
@@ -479,7 +465,6 @@ dirattr_add( filehdr_t *fhdrp )
 	return dah;
 }
 
-#ifdef EXTATTR
 void
 dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 {
@@ -760,7 +745,6 @@ dirattr_cb_extattr( dah_t dah,
 
 	return BOOL_TRUE;
 }
-#endif /* EXTATTR */
 
 void
 dirattr_update( dah_t dah, filehdr_t *fhdrp )
@@ -823,11 +807,10 @@ dirattr_update( dah_t dah, filehdr_t *fhdrp )
 	dirattr.d_ctime = ( time32_t )fhdrp->fh_stat.bs_ctime.tv_sec;
 	dirattr.d_xflags = fhdrp->fh_stat.bs_xflags;
 	dirattr.d_extsize = ( u_int32_t )fhdrp->fh_stat.bs_extsize;
+	dirattr.d_projid = fhdrp->fh_stat.bs_projid;
 	dirattr.d_dmevmask = fhdrp->fh_stat.bs_dmevmask;
 	dirattr.d_dmstate = ( u_int32_t )fhdrp->fh_stat.bs_dmstate;
-#ifdef EXTATTR
 	dirattr.d_extattroff = DIRATTR_EXTATTROFFNULL;
-#endif /* EXTATTR */
 
 	/* write the dirattr
 	 */
@@ -905,6 +888,13 @@ dirattr_get_extsize( dah_t dah )
 	return dtp->dt_cached_dirattr.d_extsize;
 }
 
+u_int32_t
+dirattr_get_projid( dah_t dah )
+{
+	dirattr_get( dah );
+	return dtp->dt_cached_dirattr.d_projid;
+}
+
 u_int32_t
 dirattr_get_dmevmask( dah_t dah )
 {
@@ -993,7 +983,6 @@ dirattr_get( dah_t dah )
 	dtp->dt_cachedh = dah;
 }
 
-#ifdef EXTATTR
 static void
 dirattr_flush( void )
 {
@@ -1064,8 +1053,6 @@ dirattr_flush( void )
 
 	dtp->dt_at_endpr = BOOL_FALSE;
 }
-#endif /* EXTATTR */
-
 
 #ifdef DIRATTRCHK
 
