@@ -1,44 +1,29 @@
 /*
- * Copyright (c) 2000 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2001 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <libxfs.h>
-#include <jdm.h>
+#include <xfs/libxfs.h>
+#include <xfs/jdm.h>
 
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <time.h>
 #include <getopt.h>
 
@@ -64,16 +49,7 @@ static FILE *mlog_fp = NULL; /* stderr */;
 static FILE *mlog_fp = NULL; /* stdout */;
 #endif /* RESTORE */
 
-intgen_t mlog_level_ss[ MLOG_SS_CNT ] = {
-	MLOG_VERBOSE,
-	MLOG_VERBOSE,
-	MLOG_VERBOSE,
-	MLOG_VERBOSE,
-#ifdef RESTORE
-	MLOG_VERBOSE,
-#endif /* RESTORE */
-	MLOG_VERBOSE
-};
+intgen_t mlog_level_ss[ MLOG_SS_CNT ];
 
 intgen_t mlog_showlevel = BOOL_FALSE;
 
@@ -87,7 +63,7 @@ static size_t mlog_streamcnt;
 
 static char mlog_levelstr[ 3 ]; 
 
-#define MLOG_SS_NAME_MAX	10
+#define MLOG_SS_NAME_MAX	15
 #ifdef DUMP
 #define PROGSTR "dump"
 #define PROGSTR_CAPS "Dump"
@@ -115,11 +91,12 @@ char *mlog_ss_names[ MLOG_SS_CNT ] = {
 	"media",	/* MLOG_SS_MEDIA */
 	"inventory",	/* MLOG_SS_INV */
 #ifdef DUMP
-	"inomap"	/* MLOG_SS_INOMAP */
+	"inomap",	/* MLOG_SS_INOMAP */
 #endif /* DUMP */
 #ifdef RESTORE
-	"tree"		/* MLOG_SS_TREE */
+	"tree",		/* MLOG_SS_TREE */
 #endif /* RESTORE */
+	"excluded_files" /* MLOG_SS_EXCLFILES */
 };
 
 static mlog_sym_t mlog_sym[ ] = {
@@ -200,7 +177,7 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 		case GETOPT_VERBOSITY:
 			if ( ! optarg || optarg[ 0 ] == '-' ) {
 				fprintf( stderr,
-					 "%s: -%c argument missing\n",
+					 _("%s: -%c argument missing\n"),
 					 progname,
 					 optopt );
 				usage( );
@@ -216,7 +193,7 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 						      &valstr );
 				if ( suboptix < 0 ) {
 					fprintf( stderr,
-						 "%s: -%c argument invalid\n",
+						 _("%s: -%c argument invalid\n"),
 						 progname,
 						 optopt );
 					usage( );
@@ -228,10 +205,10 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 				if ( suboptix < MLOG_SS_CNT ) {
 					if ( ! valstr ) {
 						fprintf( stderr,
-							 "%s: -%c subsystem "
+							 _("%s: -%c subsystem "
 							 "subargument "
 							 "%s requires a "
-							 "verbosity value\n",
+							 "verbosity value\n"),
 							 progname,
 							 optopt,
 						mlog_ss_names[ suboptix ] );
@@ -244,9 +221,9 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 				} else {
 					if ( valstr ) {
 						fprintf( stderr,
-							 "%s: -%c argument "
+							 _("%s: -%c argument "
 							 "does not require "
-							 "a value\n",
+							 "a value\n"),
 							 progname,
 							 optopt );
 						usage( );
@@ -258,8 +235,8 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 				}
 				if ( mlog_level_ss[ ssix ] < 0 ) {
 					fprintf( stderr,
-						 "%s: -%c argument "
-						 "invalid\n",
+						 _("%s: -%c argument "
+						 "invalid\n"),
 						 progname,
 						 optopt );
 					usage( );
@@ -344,6 +321,31 @@ mlog_unlock( void )
 	qlock_unlock( mlog_qlockh );
 }
 
+/*
+ * Override the -v option.
+ * Useful for debugging at particular points
+ * where doing it program-wide would produce
+ * too much output.
+ */
+void
+mlog_override_level( intgen_t levelarg )
+{
+	intgen_t level;
+	ix_t ss; /* SubSystem */
+
+	level = levelarg & MLOG_LEVELMASK;
+	ss = ( ix_t )( ( levelarg & MLOG_SS_MASK ) >> MLOG_SS_SHIFT );
+
+	if (ss == MLOG_SS_GEN) { /* do level for all subsys */  
+	    for (ss = 0 ; ss < MLOG_SS_CNT ; ss++ ) {
+		mlog_level_ss[ ss ] = level;
+	    }
+	}
+	else { /* do a particular subsys */
+	    mlog_level_ss[ ss ] = level;
+	}
+}
+
 void
 mlog( intgen_t levelarg, char *fmt, ... )
 {
@@ -409,7 +411,7 @@ mlog_va( intgen_t levelarg, char *fmt, va_list args )
 		}
 		if ( streamix != -1 && mlog_streamcnt > 1 ) {
 			fprintf( mlog_fp,
-				 "%s%s%s%s: drive %d: ",
+				 _("%s%s%s%s: drive %d: "),
 				 progname,
 				 mlog_tsstr,
 				 mlog_ssstr,
@@ -462,7 +464,7 @@ struct rv_map {
 
 static struct rv_map
 rvs[_RV_NUM] = {
-	/* Return Code	Displayed Code	Explanation */
+       /* Return Code	Displayed Code	Explanation */
 	{ RV_OK,	"OK",		"success" }, 
 	{ RV_NOTOK,	"ERASE_FAILED",	"media erase request denied" },
 	{ RV_NOMORE,	"NOMORE",	"no more work to do" },
@@ -486,7 +488,9 @@ rvs[_RV_NUM] = {
 	{ RV_INCOMPLETE,"INCOMPLETE",	"the " PROGSTR " is incomplete" },
 	{ RV_KBD_INTR,	"KEYBOARD_INTERRUPT", "keyboard interrupt" },
 	{ RV_INV,	"INVENTORY",	"error updating session inventory" },
-	{ RV_NONE,	"NONE",		"no error code specified" },
+	{ RV_USAGE,	"USAGE_ONLY",	"printing usage only" },
+	{ RV_EXISTS,	"EXISTS",	"file or directory already exists" },
+	{ RV_NONE,	"NONE",		"no error code" },
 	{ RV_UNKNOWN,	"UNKNOWN",	"unknown error" },
 };
 
@@ -566,7 +570,7 @@ _mlog_exit( const char *file, int line, int exit_code, rv_t rv )
 	rvp = rv_getdesc(rv);
 
 
-	mlog( MLOG_DEBUG | MLOG_NOLOCK | MLOG_BARE,
+	mlog( MLOG_DEBUG | MLOG_NOLOCK,
 	      "%s: %d: mlog_exit called: "
 	      "exit_code: %s return: %s (%s)\n",
 	      file, line,
@@ -574,7 +578,7 @@ _mlog_exit( const char *file, int line, int exit_code, rv_t rv )
 	      rvp->rv_string, rvp->rv_desc);
 
 	if (rv < 0 || rv >= _RV_NUM) {
-		mlog( MLOG_DEBUG | MLOG_ERROR | MLOG_NOLOCK | MLOG_BARE,
+		mlog( MLOG_DEBUG | MLOG_ERROR | MLOG_NOLOCK,
 		      "mlog_exit(): bad return code");
 		return exit_code;
 	}
@@ -627,14 +631,14 @@ _mlog_exit_hint( const char *file, int line, rv_t rv )
 	pid = getpid();
 	rvp = rv_getdesc(rv);
 	
-	mlog( MLOG_DEBUG | MLOG_NOLOCK | MLOG_BARE,
+	mlog( MLOG_DEBUG | MLOG_NOLOCK,
 	      "%s: %d: mlog_exit_hint called: "
 	      "hint: %s (%s)\n",
 	      file, line,
 	      rvp->rv_string, rvp->rv_desc);
 
 	if (rv < 0 || rv >= _RV_NUM) {
-		mlog( MLOG_DEBUG | MLOG_ERROR | MLOG_NOLOCK | MLOG_BARE,
+		mlog( MLOG_DEBUG | MLOG_ERROR | MLOG_NOLOCK,
 		      "mlog_exit_hint(): bad return code");
 		return;
 	}
@@ -660,25 +664,30 @@ mlog_get_hint( void )
 	/* REFERENCED */
 	bool_t ok;
 	rv_t hint;
-	
+
+	if (getpid() == parentpid)
+		return mlog_main_exit_hint;
+
 	ok = stream_get_exit_status(getpid(), states, N(states),
 				    NULL, NULL, NULL, NULL, &hint);
-	assert(ok);
+	ASSERT(ok);
 	return hint;
 }
 
-bool_t
-is_incomplete(rv_t rv)
-{
-    int j;
-    rv_t errors[] = { RV_CORRUPT, RV_INCOMPLETE, RV_EOD, RV_EOF, RV_EOM };
 
-    for (j = 0; j < N(errors); j++)
-	if (rv == errors[j])
-	    return BOOL_TRUE;
+#define IS_INCOMPLETE(rv)			\
+	((rv) == RV_CORRUPT ||			\
+	 (rv) == RV_INCOMPLETE ||		\
+	 (rv) == RV_EOD ||			\
+	 (rv) == RV_EOF ||			\
+	 (rv) == RV_EOM)
+
+#define VALID_EXIT_CODE(code)			\
+	((code) == EXIT_NORMAL ||		\
+	 (code) == EXIT_ERROR ||		\
+	 (code) == EXIT_INTERRUPT ||		\
+	 (code) == EXIT_FAULT)
 
-    return BOOL_FALSE;
-}
 
 void
 mlog_exit_flush(void)
@@ -689,6 +698,7 @@ mlog_exit_flush(void)
 	stream_state_t states[] = { S_RUNNING, S_ZOMBIE };
 	bool_t incomplete = BOOL_FALSE;
 	bool_t quit = BOOL_FALSE;
+	bool_t interrupt = BOOL_FALSE;
 	const char *status_str;
 	rv_t rv;
 
@@ -702,7 +712,7 @@ mlog_exit_flush(void)
 	if (npids > 0) {
 
 		/* print the state of all the streams */
-		fprintf(mlog_fp, "%s: " PROGSTR_CAPS " Summary:\n", progname);
+		fprintf(mlog_fp, _("%s: %s Summary:\n"), progname, PROGSTR_CAPS );
 
 		for (i = 0; i < npids; i++) {
 			stream_state_t state;
@@ -720,53 +730,55 @@ mlog_exit_flush(void)
 						    &exit_code,
 						    &exit_return,
 						    &exit_hint);
-			assert(ok);
+			ASSERT(ok);
 
 			/* hint takes priority over return */
 			rv = (exit_hint != RV_NONE) ? exit_hint : exit_return;
-			if (rv != RV_NONE) {
-				/* then an mlog_*() call was made otherwise we
-				 * don't know anything about it...
-				 */
-				rvp = rv_getdesc(rv);
-				fprintf(mlog_fp,
-					"%s:   stream %d (pid %d) %s "
-					"%s (%s)\n",
-					progname,
-					streamix,
-					pids[i],
-					drivepp[streamix]->d_pathname,
-					rvp->rv_string,
-					rvp->rv_desc);
-			}
+
+			/* print status of this stream */
+			rvp = rv_getdesc(rv);
+			fprintf(mlog_fp,
+				_("%s:   stream %d (pid %d) %s "
+				"%s (%s)\n"),
+				progname,
+				streamix,
+				pids[i],
+				drivepp[streamix]->d_pathname,
+				rvp->rv_string,
+				rvp->rv_desc);
 
 			/* If the following conditions are true for any stream
 			 * then they are true for the entire dump/restore.
 			 */
-			if (! incomplete && is_incomplete(rv))
-			    incomplete = BOOL_TRUE;
-
-			if (! quit && rv == RV_QUIT)
-			    quit = BOOL_TRUE;
+			if (rv == RV_INTR) interrupt = BOOL_TRUE;
+			else if (rv == RV_QUIT) quit = BOOL_TRUE;
+			else if (IS_INCOMPLETE(rv)) incomplete = BOOL_TRUE;
 		}
 	}
 
-	/* Check return codes for the main process
+	/* Also check return codes for the main process
 	 */
-	rv = (mlog_main_exit_hint != RV_NONE)
-	    ? mlog_main_exit_hint : mlog_main_exit_return;
+	rv = (mlog_main_exit_hint != RV_NONE) ? mlog_main_exit_hint
+	    : mlog_main_exit_return;
 
-	if (! incomplete && is_incomplete(rv))
-	    incomplete = BOOL_TRUE;
+	if (rv == RV_INTR) interrupt = BOOL_TRUE;
+	else if (rv == RV_QUIT) quit = BOOL_TRUE;
+	else if (IS_INCOMPLETE(rv)) incomplete = BOOL_TRUE;
 
-	if (! quit && rv == RV_QUIT)
-	    quit = BOOL_TRUE;
-		
-	/* now print the overall state of the dump/restore */
-	if (incomplete) status_str = "INCOMPLETE";
+	/* if we don't have an exit code here there is a problem */
+	ASSERT(VALID_EXIT_CODE(mlog_main_exit_code));
+	if (interrupt) status_str = "INTERRUPT";
 	else if (quit) status_str = "QUIT";
+	else if (incomplete) status_str = "INCOMPLETE";
+#ifdef NDEBUG
+	/* We should never get here, but if we do make sure we don't die
+	   horribly when not running debug. */
+	else if (! VALID_EXIT_CODE(mlog_main_exit_code)) status_str = "UNKNOWN";
+#endif /* NDEBUG */
 	else status_str = exit_strings[mlog_main_exit_code];
-	fprintf(mlog_fp, "%s: " PROGSTR_CAPS " Status: %s\n", progname, status_str);
+
+	/* now print the overall state of the dump/restore */
+	fprintf(mlog_fp, "%s: %s Status: %s\n", progname, PROGSTR_CAPS, status_str);
 	fflush(mlog_fp);
 }
 
