@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2001 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2001 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 #ifndef TREE_H
 #define TREE_H
@@ -46,14 +32,16 @@ extern bool_t tree_init( char *hkdir,
 			 size64_t vmsz,
 			 bool_t fullpr,
 			 bool_t restoredmpr,
-			 bool_t largewindowpr );
+			 bool_t largewindowpr,
+			 bool_t dstdirisxfspr );
 
 /* tree_sync - synchronizes with an existing tree abstraction
  */
 extern bool_t tree_sync( char *hkdir,
 			 char *dstdir,
 			 bool_t toconlypr,
-			 bool_t fullpr);
+			 bool_t fullpr,
+			 bool_t dstdirisxfspr );
 
 
 /* tree_begindir - begins application of dumped directory to tree.
@@ -118,7 +106,6 @@ extern bool_t tree_setattr( char *path );
 extern bool_t tree_delorph( void );
 extern bool_t tree_subtree_inter( void );
 
-#ifdef EXTATTR
 extern bool_t tree_extattr( bool_t ( * cbfunc )( char *path, dah_t dah ),
 			    char *path );
 	/* does a depthwise bottom-up traversal of the tree, calling
@@ -128,6 +115,7 @@ extern bool_t tree_extattr( bool_t ( * cbfunc )( char *path, dah_t dah ),
 	 * callback returns FALSE. returns FALSE if operator requests
 	 * an interrupt.
 	 */
-#endif /* EXTATTR */
+
+extern void tree_update_node_dah(xfs_ino_t, gen_t, dah_t);
 
 #endif /* TREE_H */
